"""Functions to manage Azure Resource Groups."""

import secrets
import string

from azure.identity import DefaultAzureCredential
from azure.mgmt.resource import ResourceManagementClient

from kina.azure.helpers import get_subscription_username


def create_resource_group(
    credential: DefaultAzureCredential,
    subscription_id: str,
    locations: list[str],
) -> str:
    """Create a new Azure resource group for Kina Clusters.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        locations (list[str]): List of Azure locations to create Kubernetes Clusters in.

    Returns:
        str: Name of the created resource group.

    """
    resource_suffix = "".join(secrets.choice(string.ascii_lowercase + string.digits) for _ in range(6))
    resource_group_name = f"kina-{resource_suffix}"
    client = ResourceManagementClient(credential, subscription_id)
    client.resource_groups.create_or_update(
        resource_group_name,
        {
            "location": locations[0],
            "tags": {
                "managed-by": "kina",
                "locations": ",".join(locations),
                "created-by": get_subscription_username(subscription_id),
            },
        },
    )
    return resource_group_name


def list_resource_groups(
    credential: DefaultAzureCredential,
    subscription_id: str,
) -> list[tuple[str, str, str]]:
    """List all Azure resource groups created by Kina.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.

    Returns:
        list[tuple[str, str, str, str]]: List of tuples containing resource group name, location, creator, and multicluster.

    """  # noqa: E501
    client = ResourceManagementClient(credential, subscription_id)
    return [
        (rg.name, rg.tags.get("locations"), rg.tags.get("created-by"))
        for rg in client.resource_groups.list()
        if rg.tags is not None and rg.tags.get("managed-by") == "kina"
    ]


def delete_resource_group(credential: DefaultAzureCredential, subscription_id: str, resource_group_name: str) -> bool:
    """Delete an Azure resource group created by Kina.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        resource_group_name (str): Name of the resource group to delete.

    Returns:
        bool: True if the resource group was deleted, False otherwise.

    """
    # TODO(@cpressland): Improve validation of Kina Resource Groups.
    # https://github.com/cpressland/kina/issues/3
    deleted = False
    if resource_group_name.startswith("kina-"):
        client = ResourceManagementClient(credential, subscription_id)
        client.resource_groups.begin_delete(resource_group_name)
        deleted = True
    return deleted



"""Helper functions for Azure CLI integration."""

import json
from pathlib import Path

from azure.identity import DefaultAzureCredential
from azure.mgmt.resource import SubscriptionClient


def get_location_availability_zones(
    credential: DefaultAzureCredential,
    subscription_id: str,
    location: str,
) -> list[str]:
    """Get the availability zones for a given Azure location.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        location (str): Azure region to check for availability zones.

    Returns:
        list[str]: List of availability zones in the specified location.

    """
    client = SubscriptionClient(credential)
    loc = next((loc for loc in client.subscriptions.list_locations(subscription_id) if loc.name == location), None)
    return [zone.logical_zone for zone in (loc.availability_zone_mappings or [])]


def get_subscription_username(subscription_id: str) -> str:
    """Get the username of the Azure account associated with the given subscription ID.

    Args:
        subscription_id (str): The Azure subscription ID.

    Returns:
        str: The username associated with the subscription ID.

    """
    try:
        azure_profile = json.loads(Path("~/.azure/azureProfile.json").expanduser().read_text())
    except json.JSONDecodeError:
        azure_profile = json.loads(Path("~/.azure/azureProfile.json").expanduser().read_text(encoding="utf-8-sig"))
    except FileNotFoundError:
        raise FileNotFoundError("Azure profile file not found. Please log in to Azure CLI.") from None

    return next(sub for sub in azure_profile["subscriptions"] if sub["id"] == subscription_id)["user"]["name"]


def get_default_subscription_id() -> str:
    """Get the default Azure subscription ID.

    Returns:
        str: The default Azure subscription ID.

    """
    try:
        azure_profile = json.loads(Path("~/.azure/azureProfile.json").expanduser().read_text())
    except json.JSONDecodeError:
        azure_profile = json.loads(Path("~/.azure/azureProfile.json").expanduser().read_text(encoding="utf-8-sig"))
    except FileNotFoundError:
        raise FileNotFoundError("Azure profile file not found. Please log in to Azure CLI.") from None

    return next(sub for sub in azure_profile["subscriptions"] if sub["isDefault"])["id"]



"""Functions to manage Azure Kubernetes Service."""

from time import sleep

from azure.identity import DefaultAzureCredential
from azure.mgmt.authorization import AuthorizationManagementClient
from azure.mgmt.containerservice import ContainerServiceClient
from azure.mgmt.network import NetworkManagementClient
from rich.progress import Progress, TaskID

from kina.azure.helpers import get_location_availability_zones


def get_latest_aks_version(credential: DefaultAzureCredential, subscription_id: str, location: str) -> str:
    """Get the available Kubernetes versions for AKS clusters.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        location (str): Azure region to check for available Kubernetes versions.

    Returns:
        list[str]: List of available Kubernetes versions.

    """
    aks_client = ContainerServiceClient(credential, subscription_id)
    versions = aks_client.managed_clusters.list_kubernetes_versions(location)
    return max(
        (patch for v in versions.values for patch in v.patch_versions),
        key=lambda ver: tuple(map(int, ver.split("."))),
    )


def create_aks_clusters(
    credential: DefaultAzureCredential,
    subscription_id: str,
    virtual_network_names: list[str],
    resource_group_name: str,
    no_cni: bool = False,
    rich_progress: Progress | None = None,
    rich_task: TaskID | None = None,
) -> list[str]:
    """Create Azure Kubernetes Service (AKS) clusters in the specified virtual networks.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        virtual_network_names (list[str]): List of virtual network names to create AKS clusters in.
        resource_group_name (str): Name of the resource group to create the AKS clusters in.
        no_cni (bool, optional): Flag to disable CNI. Defaults to False.
        rich_progress (Progress | None, optional): Optional Rich Progress instance for progress tracking. Defaults to None.
        rich_task (TaskID | None, optional): Optional task ID for progress tracking. Defaults to None.

    Returns:
        list[str]: List of AKS cluster names created.

    """  # noqa: E501
    aks_client = ContainerServiceClient(credential, subscription_id)
    network_client = NetworkManagementClient(credential, subscription_id)
    clusters = []

    for virtual_network_name in virtual_network_names:
        location = network_client.virtual_networks.get(resource_group_name, virtual_network_name).location
        vnet = network_client.virtual_networks.get(resource_group_name, virtual_network_name)
        subnet = network_client.subnets.get(resource_group_name, virtual_network_name, "kube_nodes")
        cluster_name = f"{resource_group_name}-{location}"
        if rich_progress and rich_task is not None:
            rich_progress.update(rich_task, description=f"Creating AKS Cluster: {cluster_name}", advance=1)
        aks_client.managed_clusters.begin_create_or_update(
            resource_group_name=resource_group_name,
            resource_name=cluster_name,
            parameters={
                "location": location,
                "dns_prefix": cluster_name,
                "identity": {"type": "SystemAssigned"},
                "nodeResourceGroup": f"{resource_group_name}-{location}-nodes",
                "kubernetesVersion": get_latest_aks_version(credential, subscription_id, location),
                "agent_pool_profiles": [
                    {
                        "name": "default",
                        "mode": "System",
                        "minCount": 2,
                        "maxCount": 6,
                        "enableAutoScaling": True,
                        "vm_size": "Standard_D2ads_v5",
                        "osSKU": "AzureLinux",
                        "vnet_subnet_id": subnet.id,
                        "osDiskSizeGB": 64,
                        "osDiskType": "Ephemeral",
                        "availabilityZones": get_location_availability_zones(credential, subscription_id, location),
                    },
                ],
                "securityProfile": {
                    "imageCleaner": { "enabled": True }
                },
                "oidcIssuerProfile": { "enabled": True },
                "networkProfile": {
                    "networkPlugin": "none" if no_cni else "azure",
                    "networkPluginMode": None if no_cni else "overlay",
                    "networkMode": "transparent",
                    "loadBalancerSku": "standard",
                    "podCidr": vnet.tags.get("pod-network"),
                    "serviceCidr": vnet.tags.get("service-network"),
                    "dnsServiceIP": vnet.tags.get("dns-service-ip"),
                    "ipFamilies": ["IPv4"],
                },
            },
        )
        clusters.append(cluster_name)
    return clusters


def configure_network_iam(
    credential: DefaultAzureCredential,
    subscription_id: str,
    resource_group: str,
    cluster_name: str,
) -> None:
    """Configure network IAM for the AKS cluster.

    Args:
        credential (DefaultAzureCredential): Azure credentials for authentication.
        subscription_id (str): Azure subscription ID.
        resource_group (str): Name of the resource group containing the AKS cluster.
        cluster_name (str): Name of the AKS cluster.

    Returns:
        None

    """
    auth_client = AuthorizationManagementClient(credential, subscription_id)
    aks_client = ContainerServiceClient(credential, subscription_id)
    for _ in range(30):
        try:
            cluster = aks_client.managed_clusters.get(resource_group, cluster_name)
            principal_id = cluster.as_dict()["identity"]["principal_id"]
            auth_client.role_assignments.create(
                scope=f"/subscriptions/{subscription_id}/resourceGroups/{resource_group}",
                role_assignment_name=principal_id,
                parameters={
                    "role_definition_id": "/providers/Microsoft.Authorization/roleDefinitions/4d97b98b-1d4f-4787-a291-c67834d212e7",  # noqa: E501
                    "principal_id": principal_id,
                },
            )
            break
        except KeyError:
            sleep(20)
            continue
